@@ -1,3 +1,25 @@
+/// An iterator over the ancestors one or more starting commits
+pub struct Ancestors<Find, Predicate, StateMut> {
+    find: Find,
+    predicate: Predicate,
+    state: StateMut,
+    mode: Parents,
+}
+
+/// Specify how to handle commit parents during traversal.
+pub enum Parents {
+    /// Traverse all parents, useful for traversing the entire ancestry.
+    All,
+    /// Only traverse along the first parent, which commonly ignores all branches.
+    First,
+}
+
+impl Default for Parents {
+    fn default() -> Self {
+        Parents::All
+    }
+}
+
 ///
 pub mod ancestors {
     use std::{
@@ -9,6 +31,8 @@ pub mod ancestors {
     use git_object::CommitRefIter;
     use quick_error::quick_error;
 
+    use crate::commit::{Ancestors, Parents};
+
     quick_error! {
         /// The error is part of the item returned by the [Ancestors] iterator.
         #[derive(Debug)]
@@ -41,11 +65,12 @@ pub mod ancestors {
         }
     }
 
-    /// An iterator over the ancestors one or more starting commits
-    pub struct Ancestors<Find, Predicate, StateMut> {
-        find: Find,
-        predicate: Predicate,
-        state: StateMut,
+    impl<Find, Predicate, StateMut> Ancestors<Find, Predicate, StateMut> {
+        /// Change our commit parent handling mode to the given one.
+        pub fn mode(mut self, mode: Parents) -> Self {
+            self.mode = mode;
+            self
+        }
     }
 
     impl<Find, StateMut> Ancestors<Find, fn(&oid) -> bool, StateMut>
@@ -108,7 +133,12 @@ pub mod ancestors {
                     }
                 }
             }
-            Self { find, predicate, state }
+            Self {
+                find,
+                predicate,
+                state,
+                mode: Default::default(),
+            }
         }
     }
 
@@ -136,6 +166,9 @@ pub mod ancestors {
                                     if was_inserted && (self.predicate)(&id) {
                                         state.next.push_back(id);
                                     }
+                                    if matches!(self.mode, Parents::First) {
+                                        break;
+                                    }
                                 }
                                 Ok(_a_token_past_the_parents) => break,
                                 Err(err) => return Some(Err(err.into())),
@@ -149,5 +182,3 @@ pub mod ancestors {
         }
     }
 }
-#[doc(inline)]
-pub use ancestors::Ancestors;
